@@ -462,8 +462,8 @@ class LazyMedia(anvil.Media):
         except AnvilWrappedError as e:
             raise _deserialise_exception(e.error_obj)
 
-    def get_url(self, is_download=True):
-        return anvil.server.call("anvil.private.get_lazy_media_url", self, is_download)
+    def get_url(self, download=True):
+        return anvil.server.call("anvil.private.get_lazy_media_url", self, download)
 
     def get_content_type(self):
         try:
@@ -979,8 +979,7 @@ def fill_out_media(json, handle_media_fn, collect_capabilities=None, remote_is_t
             _json = None
         elif 'numpy' in sys.modules and hasattr(sys.modules['numpy'], 'generic') and isinstance(_json, sys.modules['numpy'].generic):
 
-            import numpy
-            _json = numpy.asscalar(_json)
+            _json = _json.item() # convert
 
         elif not (isinstance(_json, string_type) or _json is None or _json is True or _json is False or isinstance(_json, (int, long_type)) or isinstance(_json, float)):
 
@@ -1283,12 +1282,14 @@ class CallContext(object):
             self.client = CallContext.ClientInfo()
             self.client.type = client.get('type')
             self.client.ip = client.get('ip')
+            self.background_task_id = client.get('background-task-id')
             l = client.get('location')
             self.client.location = CallContext.Location(l) if l else None
             self.remote_caller = CallContext.StackFrame(call_stack[0] if call_stack else client)
         else:
             self.client = None
             self.remote_caller = None
+            self.background_task_id = None
 
     __init__ = _setup
 
@@ -1426,7 +1427,14 @@ def http_endpoint(path, require_credentials=False, authenticate_users=False, aut
                     d["Access-Control-Allow-Origin"] = same_app_alternate_origin
                 if enable_cors or same_app_alternate_origin and \
                         "access-control-allow-headers" not in [h.lower() for h in d.keys()]:
-                    d["Access-Control-Allow-Headers"] = "content-type"
+                    allow_headers = ["content-type"]
+                    for k in headers.keys():
+                        if k.lower() == "access-control-request-headers":
+                            hs = [h.strip().lower() for h in headers[k].split(",")]
+                            allow_headers += [h for h in hs if h not in allow_headers]
+
+                    d["Access-Control-Allow-Headers"] = ", ".join(allow_headers)
+
                 return d
 
             if method not in methods:
