@@ -3,6 +3,9 @@ import os
 import sys
 import anvil
 import tempfile
+import io
+
+open_ = open
 
 class TempFile():
 
@@ -14,7 +17,7 @@ class TempFile():
     def __enter__(self):
         self._filename = tempfile.gettempdir() + os.sep + "".join([random.choice("1234567890abcdefghijklmnopqrstuvwxyz") for i in range(32)])
         if self._media is not None:
-            with open(self._filename, "wb") as f:
+            with open_(self._filename, "wb") as f:
                 f.write(self._media.get_bytes())
         return self._filename
 
@@ -30,10 +33,15 @@ class TempFile():
 
 #!defFunction(anvil.media,%anvil.Media instance,filename,[mime_type],[name])!2: "Creates a Media object from the given file." ["from_file"]
 def from_file(filename, mime_type=None, name=None):
-    with open(filename, "rb") as f:
+    with open_(filename, "rb") as f:
         return anvil.BlobMedia(mime_type, f.read(), name=(name or filename.split(os.sep)[-1]))
 
 #!defFunction(anvil.media,_,media,filename)!2: "Write a Media object to the given file" ["write_to_file"]
 def write_to_file(media, filename):
-    with open(filename, "wb") as f:
+    with open_(filename, "wb") as f:
         f.write(media.get_bytes())
+
+
+#!defFunction(anvil.media,%BytesIO, media)!2: "Open a media file as Python BytesIO object" ["open"]
+def open(media):
+    return io.BytesIO(media.get_bytes())
\ No newline at end of file
