@@ -399,6 +399,8 @@ _server._do_call = _do_call
 
 
 def call(fn_name, *args, **kwargs):
+    if not isinstance(fn_name, str):
+        raise TypeError("first argument to anvil.server.call() must be as str, got '" + type(fn_name).__name__ + "'")
     try:
         return _do_call(args, kwargs, fn_name=fn_name)
     except _server.AnvilWrappedError as e:
