@@ -12,6 +12,14 @@ _make_refs = None  # for circular imports
 _auto_create_is_enabled = None
 
 
+def _copy(so):
+    if type(so) is list:
+        return [_copy(o) for o in so]
+    if type(so) is dict:
+        return {k: _copy(v) for k, v in so.items()}
+    return so
+
+
 @anvil.server.portable_class
 class Row(BaseRow):
     @classmethod
@@ -262,7 +270,12 @@ class Row(BaseRow):
             return  # no uncached values
 
         table_data = anvil.server.call(PREFIX + "fetch", self._cap, uncached_keys)
-        self._unpack(table_data, table_data[self._view_key]["rows"][self._id])
+        rows = table_data[self._view_key]["rows"]
+        row_data = rows[self._id]
+        # Replace the compact row data with this Row instance
+        # so circular references don't clobber the data while we're unpacking.
+        rows[self._id] = self
+        self._unpack(table_data, row_data)
 
     def _walk_local_items(self, items, missing=NOT_FOUND):
         # We are about to put local items in the cache
@@ -276,7 +289,7 @@ class Row(BaseRow):
             if val is NOT_FOUND:
                 continue
             else:
-                rv[name] = val
+                rv[name] = _copy(val)
             if val is UNCACHED or val is None:
                 continue
             elif type == DATETIME:
@@ -332,9 +345,9 @@ class Row(BaseRow):
                 # try to force fetch this key - incase we have a bad spec - i.e auto-columns
                 self._fill_cache([key])
         else:
-            return hit
+            return _copy(hit)
         try:
-            return self._cache[key]
+            return _copy(self._cache[key])
         except KeyError:
             raise NoSuchColumnError("No such column '" + key + "'")
 
@@ -372,7 +385,9 @@ class Row(BaseRow):
         # Find all the remaining columns
         num_remaning = len(cols) - len(cached_printable_cols)
 
-        vals = ", ".join("{}={}".format(name, None if val is None else meth(val)) for name, meth, val in cached_printable_cols)
+        vals = ", ".join(
+            "{}={}".format(name, None if val is None else meth(val)) for name, meth, val in cached_printable_cols
+        )
 
         if not num_remaning:
             and_more = ""
@@ -409,15 +424,22 @@ class Row(BaseRow):
 
     def items(self):
         self._fill_cache()
-        return self._cache.items()
+        return _copy(self._cache).items()
 
     def values(self):
         self._fill_cache()
-        return self._cache.values()
+        return _copy(self._cache).values()
 
-    def update(self, other=(), **new_items):
-        if other != ():
-            new_items = dict(other, **new_items)
+    def update(*args, **new_items):
+        # avoid name conflicts with columns, could use (self, other, /, **kws)
+        # but positioin only args not available in py2/Skulpt
+        if not args:
+            raise TypeError("method 'update' of 'Row' object needs an argument")
+        elif len(args) > 2:
+            raise TypeError("expected at most 1 argument, got %d" % (len(args) - 1))
+        elif len(args) == 2:
+            new_items = dict(args[1], **new_items)
+        self = args[0]
         if not new_items:
             # backwards compatability hack
             self._clear_cache()
@@ -446,6 +468,7 @@ class Row(BaseRow):
     def refresh(self, fetch=None):
         if fetch is not None:
             from ..query import fetch_only
+
             if not isinstance(fetch, fetch_only):
                 raise TypeError("the second argument to refresh should be a q.fetch_only() object")
             fetch = fetch.spec
@@ -475,6 +498,6 @@ class RowIterator:
         key = item[0]
         # we rely here on the _cache keys not changing during iteration
         # which works since we've filled it with UNCACHED values that match our expected keys
-        return (key, self._row._cache[key])
+        return (key, _copy(self._row._cache[key]))
 
     next = __next__
