@@ -1,7 +1,44 @@
 import anvil.server
 from anvil import *
-from ..exceptions import AuthenticationFailed
+from ..exceptions import AuthenticationFailed, MFAException
 from ..config import get_client_config
+from anvil.js import window
+
+def replace(s, pattern, replacement):
+    return window.String.prototype.replaceAll.call(s, window.RegExp(pattern, "g"), replacement)
+
+class PhoneBox(TextBox):
+
+    def __init__(self, **properties):
+        TextBox.__init__(self, **properties)
+        self.valid_number = None
+        self.type="tel"
+        self.add_event_handler("focus", self.on_focus)
+        self.add_event_handler("lost_focus", self.on_blur)
+        self.add_event_handler("pressed_enter", self.on_blur)
+
+    def on_focus(self, **e):
+        self.placeholder = "(123) 456 7890" if window.navigator.language == "en-US" else "+1 000 0..."
+
+    def on_blur(self, **e):
+        self.validate()
+
+    def validate(self):
+        leadingPlus = self.text.startswith("+")
+        self.text = replace(self.text, "[^0-9]", "")
+        if leadingPlus:
+            if len(self.text) > 3: # What's a valid minimum length?
+                self.text = "+" + self.text
+                self.valid_number = self.text
+            else:
+                self.valid_number = None
+        else:
+            if len(self.text) == 10:
+                self.valid_number = "+1" + self.text
+                self.text = "(" + self.text[0:3] + ") " + self.text[3:6] + " " + self.text[6:]
+            else:
+                self.valid_number = None
+
 
 #!defFunction(anvil.users,_,email_address)!2: "Send a two-factor authentication reset email to the specified user." ["send_mfa_reset_email"]
 def send_mfa_reset_email(email):
@@ -69,12 +106,17 @@ else:
         mfa_types = get_enabled_mfa_types()
         selected_mfa_type = None
         mfa_methods = {}
+        password_error = None
+        password_box = TextBox(placeholder="Current password", align="center", hide_text=True)
 
         while True:
             mfa_panel = LinearPanel()
 
-            password_box = TextBox(placeholder="Current password", align="center", hide_text=True)
-            password_box.set_event_handler("pressed_enter", lambda **e: mfa_panel.raise_event("x-close-alert"))
+            if require_password and not password_box.text:
+                password_box.remove_from_parent()
+                mfa_panel.add_component(password_box)
+                if password_error:
+                    mfa_panel.add_component(Label(foreground="red", align="center", spacing_below="none", text=password_error))
 
             if not selected_mfa_type:
 
@@ -121,31 +163,28 @@ else:
 
                     totp_box = TextBox(placeholder="Enter 6-digit code", align="center", font="monospace")
                     totp_box.set_event_handler("show", lambda **e: totp_box.focus())
-                    totp_box.set_event_handler("pressed_enter", lambda **e: password_box.focus() if require_password else mfa_panel.raise_event("x-close-alert"))
+                    totp_box.set_event_handler("pressed_enter", lambda **e: mfa_panel.raise_event("x-close-alert"))
                     mfa_panel.add_component(totp_box)
-                    if require_password:
-                        mfa_panel.add_component(password_box)
 
                 elif selected_mfa_type == "twilio-verify":
                     if not mfa_methods.get('twilio-verify'):
-                        phone_box = TextBox(placeholder="Enter your phone number", align="center")
-                        phone_box.set_event_handler("pressed_enter", lambda **e: mfa_panel.raise_event("x-close-alert"))
-                        phone_box.set_event_handler("show", lambda **e: phone_box.focus())
+                        phone_box = PhoneBox(placeholder="Enter your phone number", align="center")
+                        phone_box.add_event_handler("pressed_enter", lambda **e: mfa_panel.raise_event("x-close-alert"))
+                        #phone_box.add_event_handler("lost_focus", lambda **e: print(phone_box.valid_number))
+                        #mfa_panel.add_component(Label(text="Enter your phone number", align="center"))
                         mfa_panel.add_component(phone_box)
                     else:
                         twilio_box = TextBox(placeholder="Enter 6-digit code", align="center", font="monospace")
-                        twilio_box.set_event_handler("pressed_enter", lambda **e: password_box.focus() if require_password else mfa_panel.raise_event("x-close-alert", value=twilio_box.text))
+                        twilio_box.set_event_handler("pressed_enter", lambda **e: mfa_panel.raise_event("x-close-alert"))
                         twilio_box.set_event_handler("show", lambda **e: twilio_box.focus())
                         mfa_panel.add_component(twilio_box)                            
 
-                        sms_link = Link(text="Resend SMS", icon="fa:commenting-o")
+                        sms_link = Link(text="Resend Text Message", icon="fa:commenting-o")
                         phone_link = Link(text="Call Me instead", icon="fa:phone")
                         sms_link.set_event_handler("click", lambda **e: mfa_panel.raise_event("x-close-alert", value='resend-sms'))
                         phone_link.set_event_handler("click", lambda **e: mfa_panel.raise_event("x-close-alert", value='call'))
                         mfa_panel.add_component(sms_link)
                         mfa_panel.add_component(phone_link)
-                        if require_password:
-                            mfa_panel.add_component(password_box)
 
             if mfa_error:
                 mfa_panel.add_component(Label(foreground="red", bold=True, spacing_below="none", text=mfa_error))
@@ -154,6 +193,8 @@ else:
             m = alert(mfa_panel, title="2-Factor Authentication", buttons=[(confirm_button_text, True, 'success')] + maybe_cancel_button, dismissible=bool(allow_cancel))
             if m == 'cancel':
                 return None, ""
+            elif require_password and not password_box.text: # TODO: Also validate password
+                password_error = "Please enter your password"
             elif m == 'back':
                 selected_mfa_type = None
                 mfa_error = None
@@ -174,27 +215,34 @@ else:
             elif selected_mfa_type == 'fido':
                 return mfa_methods['fido'], password_box.text
             elif selected_mfa_type == 'twilio-verify':
+                mfa_error = None
                 if m == 'resend-sms':
                     channel = "sms"
                     mfa_methods['twilio-verify'] = None
+                    mfa_error = "Text message resent"
                 elif m == 'call':
                     channel = "call"
                     mfa_methods['twilio-verify'] = None
+                    mfa_error = "Calling you now"
                 else:
                     channel = "sms"
 
                 if not mfa_methods.get('twilio-verify'):
-                    if phone_box.text:
-                        mfa_methods['twilio-verify'] = generate_twilio_mfa_method(phone_box.text)
-                        send_twilio_token(mfa_methods['twilio-verify'], channel)
-                        mfa_error = None
+                    if phone_box.valid_number:
+                        mfa_methods['twilio-verify'] = generate_twilio_mfa_method(phone_box.valid_number)
+                        try:
+                            send_twilio_token(mfa_methods['twilio-verify'], channel)
+                        except MFAException as e:
+                            mfa_error = e.message
                     else:
-                        mfa_error = "Please enter your phone number."
+                        mfa_error = "Please enter a valid phone number."
                 else:
                     if check_twilio_token(mfa_methods['twilio-verify'], twilio_box.text):
                         return mfa_methods['twilio-verify'], password_box.text
                     else:
-                        mfa_error = "Incorrect code entered. Please try again."                
+                        mfa_error = "Incorrect code entered. Please try again."
+            else:
+                mfa_error = None
 
 
     #!defFunction(anvil.users.mfa,!,email_address, password)!2: "Display a form to collect two-factor authentication credentials from the user currently logging in by passing the function their email and password." ["mfa_login_with_form"]
@@ -204,6 +252,16 @@ else:
         mfa_types = get_available_mfa_types(email, password)
 
         maybe_login_button = []
+
+        error_label = Label(foreground="red", bold=True, spacing_below="none",visible=False)
+        mfa_panel.add_component(error_label)
+        def error(text):
+            error_label.text = text
+            error_label.visible = True
+
+        def clear_error():
+            error_label.visible = False
+            error_label.text = ""
         
         if 'totp' in mfa_types:
             totp_box = TextBox(placeholder="Enter 6-digit code", align="center", font="monospace")
@@ -230,19 +288,28 @@ else:
         if 'twilio-verify' in mfa_types:
             
             def use_phone(**e):
-                anvil.server.call("anvil.private.users.twilio.send_verification_token", None, "call")
-            def use_sms(**e):
-                anvil.server.call("anvil.private.users.twilio.send_verification_token", None, "sms")
+                error("Calling you now")
+                try:
+                    send_twilio_token(None, "call")
+                except MFAException as e:
+                    error(e.message)
+            def use_sms(display_sent=True, **e):
+                if display_sent:
+                    error("Text message resent")
+                try:
+                    send_twilio_token(None, "sms")
+                except MFAException as e:
+                    error(e.message)
 
-            use_sms()
+            use_sms(False)
 
             twilio_box = TextBox(placeholder="Enter 6-digit code", align="center", font="monospace")
             twilio_box.set_event_handler("pressed_enter", lambda **e: mfa_panel.raise_event('x-close-alert', value=get_twilio_mfa_login(twilio_box.text)))
             twilio_box.set_event_handler("show", lambda **e: twilio_box.focus())
-            mfa_panel.add_component(Label(text="Please enter the 6-digit code we sent to your phone"))
+            mfa_panel.add_component(Label(text="Please enter the 6-digit code we sent to the phone number we have on file"))
             mfa_panel.add_component(twilio_box)
 
-            sms_link = Link(text="Resend SMS", icon="fa:commenting-o")
+            sms_link = Link(text="Resend Text Message", icon="fa:commenting-o")
             phone_link = Link(text="Call Me instead", icon="fa:phone")
             sms_link.set_event_handler("click", use_sms)
             phone_link.set_event_handler("click", use_phone)
